@@ -42,8 +42,10 @@
 #include <glib/gstdio.h>
 #include <glib-object.h>
 
+#ifdef HAVE_LIBXKLAVIER
 #include <libxklavier/xklavier.h>
 #include <X11/Xlib.h> /* for Display */
+#endif
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -540,6 +542,7 @@ get_session_command_for_file (const char *file,
         if (exec != NULL) {
                 res = is_prog_in_path (exec);
                 g_free (exec);
+                exec = NULL;
 
                 if (! res) {
                         g_debug ("GdmSessionDirect: Command not found: %s",
@@ -601,27 +604,30 @@ get_default_language_name (GdmSessionDirect *session)
 static char *
 get_system_default_layout (GdmSessionDirect *session)
 {
-    char *result;
-    Display *display;
+    char *result = NULL;
+#ifdef HAVE_LIBXKLAVIER
+    static XklEngine *engine = NULL;
     
-    result = NULL;    
-    display = XOpenDisplay (session->priv->display_name);
-    if (display) {
-        XklConfigRec *config;
-        XklEngine *engine = xkl_engine_get_instance (display);
-        if (engine)
-        {
+    if (engine == NULL) {
+	    Display *display = XOpenDisplay (session->priv->display_name);
+	    if (display != NULL) {
+		    engine = xkl_engine_get_instance (display);
+	    }
+	    /* do NOT call XCloseDisplay (display) here;
+	     * xkl_engine_get_instance() is a singleton which saves the display */
+    }
+    
+    if (engine != NULL) {
             XklConfigRec *config = xkl_config_rec_new ();
             if (xkl_config_rec_get_from_server (config, engine) && config->layouts && config->layouts[0]) {
                     if (config->variants && config->variants[0] && config->variants[0][0])
-			    result = g_strdup_printf("%s\t%s", config->layouts[0], config->variants[0]);
+                            result = g_strdup_printf("%s\t%s", config->layouts[0], config->variants[0]);
                     else
-			    result = g_strdup (config->layouts[0]);
+                            result = g_strdup (config->layouts[0]);
             }
             g_object_unref (config);
-        }
-        XCloseDisplay (display);
     }
+#endif
 
     if (!result)
         result = g_strdup ("us");    
